#include "properties_observer_filter.h"

// The code behind the macro creates a plugin and the main entries to the plugin DLL or shared object.
// The cPropertyFilter will be available through the plugins class factory.
ADTF_PLUGIN("Properties Filter Plugin",
            cPropertiesObserverFilter);

cPropertiesObserverFilter::cPropertiesObserverFilter()
{
    // first we get a pointer to our properties.
    object_ptr<IProperties> pProperties;
    THROW_IF_FAILED(GetProperties(pProperties));

    // we set some properties directly without the use of any Property Variables.
    // mind that observing changes to Property Variables work just the same.
    adtf::util::cString strProp1 = "observed_property_1";
    adtf::util::cString strProp2 = "observed_property_2";
    adtf::util::cString strSubPropertySeparator = "/";
    set_property<tInt>(*pProperties, strProp1, 10);
    set_property_by_path<adtf::util::cString>(*pProperties, strProp1 + strSubPropertySeparator + adtf::base::giant::detail::tReservedProperties::strDescription, "Property to observe");
    set_property<tInt>(*pProperties, strProp2, 20);
    set_property_by_path<adtf::util::cString>(*pProperties, strProp2 + strSubPropertySeparator + adtf::base::giant::detail::tReservedProperties::strDescription, "Another property to observe");

    // now register ourselfs as an observer.
    THROW_IF_FAILED(pProperties->RegisterPropertyObserver("observed_property_1", *this));
    THROW_IF_FAILED(pProperties->RegisterPropertyObserver("observed_property_2", *this));

    // set basic information about the component itself and purpose
    SetDescription("This filter shows how to observe property changes.");
}

// This method will be called for each change to a Property.
void cPropertiesObserverFilter::Notify(const IProperty& oProperty)
{
    // get the name of the property
    cString strPropertyName;
    oProperty.GetName(adtf_string_intf(strPropertyName));

    // we know our properties are of type int, so we use a helper object to access the value:
    property<int> oChangedProperty;
    oChangedProperty.Set(oProperty);

    LOG_INFO("Property '%s' has been changed to '%d'", strPropertyName.GetPtr(), oChangedProperty.GetValueT());
}
